@@ -82,8 +82,8 @@ const struct RB_ENTRY(node) *nextp;
 struct RB_ENTRY(tree) {
 #ifndef RB_CUSTOMIZE
 		/* comparison routine */
-int (*rb_cmp)(const void *, const void *, const void *);
-		/* config data to be passed to rb_cmp */
+int (*rb_cmp)(const char*, const char*, const void*);
+/* config data to be passed to rb_cmp */
 const void *rb_config;
 		/* root of tree */
 #endif /* RB_CUSTOMIZE */
@@ -91,8 +91,8 @@ struct RB_ENTRY(node) *rb_root;
 };
 
 #ifndef RB_CUSTOMIZE
-RB_STATIC struct RB_ENTRY(tree) *rbinit(int (*)(const void *, const void *, const void *),
-		 const void *);
+RB_STATIC struct RB_ENTRY(tree) *
+    rbinit(int (*)(const char*, const char*, const void*), const void*);
 #else
 RB_STATIC struct RB_ENTRY(tree) *RB_ENTRY(init)(void);
 #endif /* RB_CUSTOMIZE */
