@@ -21,6 +21,7 @@
 #include <string.h>
 #include <syslog.h>
 #include <unistd.h>
+#include <libgen.h>
 
 #include <inotifytools/inotify.h>
 #include <inotifytools/inotifytools.h>
@@ -54,7 +55,7 @@ static bool parse_opts(int* argc,
 		       int* fanotify,
 		       bool* filesystem);
 
-void print_help();
+void print_help(const char *tool_name);
 
 static const char* csv_escape_len(const char* string, size_t len) {
 	static char csv[MAX_STRLEN + 1];
@@ -169,7 +170,7 @@ int main(int argc, char** argv) {
 	int quiet = 0;
 	long timeout = BLOCKING_TIMEOUT;
 	int recursive = 0;
-	int fanotify = DEFAULT_FANOTIFY_MODE;
+	int fanotify = 0;
 	bool filesystem = false;
 	bool csv = false;
 	bool dodaemon = false;
@@ -186,6 +187,11 @@ int main(int argc, char** argv) {
 	bool no_newline = false;
 	int fd, rc;
 
+	if ((argc > 0) && (strncmp(basename(argv[0]), "fsnotify", 8) == 0)) {
+		// Default to fanotify for the fsnotify* tools.
+		fanotify = 1;
+	}
+
 	// Parse commandline options, aborting if something goes wrong
 	if (!parse_opts(&argc, &argv, &events, &monitor, &quiet, &timeout,
 			&recursive, &csv, &dodaemon, &sysl, &no_dereference,
@@ -204,16 +210,16 @@ int main(int argc, char** argv) {
 	if (timefmt)
 		inotifytools_set_printf_timefmt(timefmt);
 	if ((exc_regex &&
-	     !inotifytools_ignore_events_by_regex(exc_regex, REG_EXTENDED)) ||
+	     !inotifytools_ignore_events_by_regex(exc_regex, REG_EXTENDED, recursive)) ||
 	    (exc_iregex && !inotifytools_ignore_events_by_regex(
-			       exc_iregex, REG_EXTENDED | REG_ICASE))) {
+			       exc_iregex, REG_EXTENDED | REG_ICASE, recursive))) {
 		fprintf(stderr, "Error in `exclude' regular expression.\n");
 		return EXIT_FAILURE;
 	}
 	if ((inc_regex && !inotifytools_ignore_events_by_inverted_regex(
-			      inc_regex, REG_EXTENDED)) ||
+			      inc_regex, REG_EXTENDED, recursive)) ||
 	    (inc_iregex && !inotifytools_ignore_events_by_inverted_regex(
-			       inc_iregex, REG_EXTENDED | REG_ICASE))) {
+			       inc_iregex, REG_EXTENDED | REG_ICASE, recursive))) {
 		fprintf(stderr, "Error in `include' regular expression.\n");
 		return EXIT_FAILURE;
 	}
@@ -396,12 +402,32 @@ int main(int argc, char** argv) {
 		}
 
 		if (quiet < 2 && (event->mask & orig_events)) {
-			if (csv) {
-				output_event_csv(event);
-			} else if (format) {
-				inotifytools_printf(event, format);
+			// Only output to stdout if the event is for a file matching our filters
+			// or if we don't have any include filters
+			if (!inc_regex && !inc_iregex) {
+				// No include filter - output everything
+				if (csv) {
+					output_event_csv(event);
+				} else if (format) {
+					inotifytools_printf(event, format);
+				} else {
+					inotifytools_printf(event, "%w %,e %f\n");
+				}
 			} else {
-				inotifytools_printf(event, "%w %,e %f\n");
+				// We have an include filter
+				bool is_dir_event = (event->mask & IN_ISDIR);
+
+				// For non-directory events, only show if they match the filter
+				// The filter is already applied by inotifytools internally
+				if (!is_dir_event) {
+					if (csv) {
+						output_event_csv(event);
+					} else if (format) {
+						inotifytools_printf(event, format);
+					} else {
+						inotifytools_printf(event, "%w %,e %f\n");
+					}
+				}
 			}
 		}
 
@@ -427,17 +453,17 @@ int main(int argc, char** argv) {
 			if ((event->mask & IN_CREATE) ||
 			    (!moved_from && (event->mask & IN_MOVED_TO))) {
 				// New file - if it is a directory, watch it
-				char* new_file =
-				    inotifytools_dirpath_from_event(event);
-				if (new_file && *new_file && isdir(new_file) &&
-				    !inotifytools_watch_recursively(new_file,
-								    events)) {
-					output_error(
-					    sysl,
-					    "Couldn't watch new directory %s: "
-					    "%s\n",
-					    new_file,
-					    strerror(inotifytools_error()));
+				char* new_file = inotifytools_dirpath_from_event(event);
+				if (new_file && *new_file && isdir(new_file)) {
+					if (!quiet) {
+						output_error(sysl, "Watching new directory %s\n", new_file);
+					}
+					if (!inotifytools_watch_recursively(new_file, events)) {
+						output_error(sysl,
+							"Couldn't watch new directory %s: %s\n",
+							new_file,
+							strerror(inotifytools_error()));
+					}
 				}
 				free(new_file);
 			}  // IN_CREATE
@@ -573,7 +599,9 @@ static bool parse_opts(int* argc,
 		switch (curr_opt) {
 			// --help or -h
 			case 'h':
-				print_help();
+				print_help(((*argc) > 0)
+					   ? basename((*argv)[0])
+					   : "<executable>");
 				// Shouldn't process any further...
 				return false;
 
@@ -592,7 +620,6 @@ static bool parse_opts(int* argc,
 				(*recursive)++;
 				break;
 
-#ifdef ENABLE_FANOTIFY
 			// --inotify or -I
 			case 'I':
 				(*fanotify) = 0;
@@ -608,7 +635,6 @@ static bool parse_opts(int* argc,
 				(*filesystem) = true;
 				(*fanotify) = 1;
 				break;
-#endif
 
 			// --csv or -c
 			case 'c':
@@ -815,13 +841,11 @@ static bool parse_opts(int* argc,
 	return (curr_opt != '?');
 }
 
-#define TOOL_NAME TOOLS_PREFIX "wait"
-
-void print_help() {
-	printf("%s %s\n", TOOL_NAME, PACKAGE_VERSION);
+void print_help(const char *tool_name) {
+	printf("%s %s\n", tool_name, PACKAGE_VERSION);
 	printf("Wait for a particular event on a file or set of files.\n");
 	printf("Usage: %s [ options ] file1 [ file2 ] [ file3 ] [ ... ]\n",
-	       TOOL_NAME);
+	       tool_name);
 	printf("Options:\n");
 	printf("\t-h|--help     \tShow this help text.\n");
 	printf(
@@ -849,7 +873,7 @@ void print_help() {
 	    "--timeout expires.\n"
 	    "\t              \tWithout this option, %s will exit after "
 	    "one event is received.\n",
-	    TOOL_NAME);
+	    tool_name);
 	printf(
 	    "\t-d|--daemon   \tSame as --monitor, except run in the "
 	    "background\n"
@@ -860,11 +884,9 @@ void print_help() {
 	    "\t-P|--no-dereference\n"
 	    "\t              \tDo not follow symlinks.\n");
 	printf("\t-r|--recursive\tWatch directories recursively.\n");
-#ifdef ENABLE_FANOTIFY
 	printf("\t-I|--inotify\tWatch with inotify.\n");
 	printf("\t-F|--fanotify\tWatch with fanotify.\n");
 	printf("\t-S|--filesystem\tWatch entire filesystem with fanotify.\n");
-#endif
 	printf(
 	    "\t--fromfile <file>\n"
 	    "\t              \tRead files to watch from <file> or `-' for "
@@ -893,7 +915,7 @@ void print_help() {
 	    "\t              \twaiting for an event for <seconds> seconds.\n"
 	    "\t              \tIf <seconds> is zero, %s will never time "
 	    "out.\n",
-	    TOOL_NAME);
+	    tool_name);
 	printf(
 	    "\t-e|--event <event1> [ -e|--event <event2> ... ]\n"
 	    "\t\tListen for specific event(s).  If omitted, all events are \n"
